# -*- coding: utf_8 -*-
from eff import *
import resource
from util import *

# \[X
res = None

def createPlane(parent, name, pos, scale):
    plane = Plane(parent)
    plane.SetTexture(res.frametex[name])
    plane.SetEffect(res.fx['plane'], 'Tech_Alpha')
    plane.SetScale(scale)
    plane.SetDir(VZ, VY)
    plane.SetPos(pos)
    return plane

class BTNum(object):
    def __init__(self, parent, fig, pos, scale, tex, val = 0):
        w = scale.x * fig / 2.0 - scale.x / 2.0
        p = pos - VX * w
        self.num = [self.createNum(parent, p + VX * scale.x * i, scale, tex) for i in range(fig -1, -1, -1)]
        self.update(val)

    def createNum(self, parent, pos, scale, tex):
        plane = Plane(parent)
        plane.SetTexture(tex)
        plane.SetEffect(res.fx['plane'], 'Tech_Alpha_Anim')
        plane.SetScale(scale)
        plane.SetDir(VZ, VY)
        plane.SetPos(pos)
        plane.SetEffectParam('patnum', 10.0)
        return plane

    def update(self, val):
        for i in self.num:
            i.SetEffectParam('patid', float(val % 10))
            val /= 10

class BTBullet(object):
    def __init__(self, parent, player):
        self.player = player
        self.fig = 30
        scale = V1 * 32.0
        pos = v(-450.0 + 14.0, -600.0 + 20.0, 5.0)
        tex = res.frametex['bulletA']
        self.num = [self.createBullet(parent, pos + VX * 30 * i, scale, tex) for i in range(self.fig)]
        self.animcnt = 0

    def createBullet(self, parent, pos, scale, tex):
        plane = Plane(parent)
        plane.SetTexture(tex)
        plane.SetEffect(res.fx['plane'], 'Tech_Alpha_Anim')
        plane.SetScale(scale)
        plane.SetDir(VZ, VY)
        plane.SetPos(pos)
        plane.SetEffectParam('patnum', 15.0)
        plane.SetColor(v(1,1,1,0))
        return plane

    def update(self):
        self.animcnt += 1
        idx = 0
        if 50 <= self.animcnt < 55:
            idx = 1
        elif 55 <= self.animcnt < 60:
            idx = 2
        elif 60 <= self.animcnt < 65:
            idx = 3
        elif 65 <= self.animcnt < 70:
            idx = 4
        elif 170 <= self.animcnt:
            self.animcnt = 0

        for i in range(self.fig):
            self.num[i].SetEffectParam('patid', float(idx) + self.player.mode * 5)
            if i <= self.player.bullet - 1:
                self.num[i].SetColor(v(1,1,1,1))
            else:
                self.num[i].SetColor(v(1,1,1,0))

class BTWind(object):
    def __init__(self, parent, wdir):
        self.wind = createPlane(parent, 'wind', v(0, 600 - 64, 0), v(128.0, 128.0, 1.0))
        self.wind.SetEffect(res.fx['plane'], 'Tech_Alpha_Anim')
        self.wind.SetEffectParam('patnum', 4.0)
        self.wroot = wdir.getRoot()
        self.wdir = wdir

    def update(self):
        up = vnorm(self.wroot.GetPos())
        self.wind.SetDir(VZ, up)
        self.wind.SetEffectParam('patid', float(self.wdir.spd))

class BTChain(object):
    def __init__(self, parent, status):
        self.status = status
        self.root = Object(parent)
        self.root.SetPos(v(540, 400, 5))
        createPlane(self.root, 'framechain', V0, v(256.0, 256.0, 1.0))
        self.chain = BTNum(self.root, 2, V0, v(64.0, 128.0, 0.0), res.frametex['num128'], self.status.chain)
        self.Fvisible = False

    def update(self):
        val = self.status.chain
        if val >= 3 and not self.Fvisible:
            self.root.AnimPos(0.4, v(366, 400, 5))
            self.Fvisible = True
        # TODO: 0ɖ߂\mFɕςĂRbu\
        if val == 0 and self.Fvisible:
            self.root.AnimPos(0.4, v(540, 400, 5))
            self.Fvisible = False
        self.chain.update(val)

class BTScore(object):
    def __init__(self, parent, status):
        self.status = status
        self.score = BTNum(parent, 10, v(-275, 538, 5), v(32.0, 64.0, 1.0), res.frametex['num64'])
        self.hiscore = BTNum(parent, 10, v(275, 538, 5), v(32.0, 64.0, 1.0), res.frametex['num64'])

    def update(self):
        self.score.update(self.status.score)
        self.hiscore.update(self.status.hiscore)

class BTachievementvement(object):
    def __init__(self, parent):
        achievementtex = ('ac9CHAIN','ac15CHAIN','ac30CHAIN','ac50CHAIN','ac99CHAIN','acFIRST9','ac3COLOR','ac5COLOR','acCYCLESHOT',)
        self.achievement = [createPlane(parent, achievementtex[i], v(-550, 425 - i * 50, 5), v(100.0, 50.0, 1.0)) for i in range(9)]
        self.Fvisible = [False for i in range(9)]
        self.localtime = [0 for i in range(9)]

    def set(self, type):
        self.Fvisible[type] = True
        self.localtime[type] = 0

    def update(self):
        for i in range(9):
            if self.Fvisible[i]:
                if self.localtime[i] == 0:
                    self.achievement[i].AnimPos(0.4, v(-400, 425 - i * 50, 5))
                if self.localtime[i] >= 180:
                    self.achievement[i].AnimPos(0.4, v(-550, 425 - i * 50, 5))
                    self.Fvisible[i] = False
                self.localtime[i] += 1


class BTOver(object):
    """
    TODO: Kɉoǉ
    """
    def __init__(self, parent, status):
        self.parent = parent
        self.status = status
        p1 = createPlane(parent, 'gameover', v(0, 0, -5), v(900.0, 1200.0, 1.0))
        tex = res.frametex['num64']
        self.score = BTNum(parent, 10, v(0, 150, -10), v(32.0, 64.0, 1.0), tex, self.status.score)
        self.hiscore = BTNum(parent, 10, v(0, -30, -10), v(32.0, 64.0, 1.0), tex, self.status.hiscore)
        self.maxchain = BTNum(parent, 2, v(0, -210, -10), v(32.0, 64.0, 1.0), tex, self.status.maxchain)
        self.maxitem = BTNum(parent, 10, v(0, -390, -10), v(32.0, 64.0, 1.0), tex, self.status.maxitem)

    def update(self):
        pass


class FrameMan(object):
    def __init__(self, parent, status, wind):
        global res
        res = resource.res
        self.parent = parent
        self.status = status

        createPlane(parent, 'frameU', v(0, 600 - 64, 5), v(900.0, 128.0, 1.0))
        self.score = BTScore(parent, status)
        self.bullet = BTBullet(parent, status.player)
        self.wind = BTWind(parent, wind)
        self.chain = BTChain(parent, status)
        self.achievement = BTachievementvement(parent)

    def over(self):
        self.over = BTOver(self.parent, self.status)

    def setAchievement(self, args): # event call
        self.achievement.set(args['type'])

    def update(self):
        self.score.update()
        self.bullet.update()
        self.wind.update()
        self.chain.update()
        self.achievement.update()

