# -*- coding: utf_8 -*-

import ConfigParser
import math

from eff import *

class InitSys():
    def __init__(self, section):
        config = ConfigParser.ConfigParser()
        config.read('config.cfg')
        self.winX = config.getint(section,"winX")
        self.winY = config.getint(section,"winY")
        self.winW = config.getint(section,"winW")
        self.winH = config.getint(section,"winH")
        self.fullScreen = config.getint(section,"fullScreen")
        self.scene = 0
        #self.winsize = v(1.0, 1.0, 1.0) * (0.5, 0.8, 1.0)[self.initSys.winsize]
        self.winsize = v(1.0, 1.0, 1.0) * self.winW / 1600.0
        del config
        self.hwnd = CreateDefaultWindow(
            title=u'東方氷禄弾 ～ Incremental Chilled Essence ～  Ver.0.6',
            x=self.winX,
            y=self.winY,
            width=self.winW,
            height=self.winH,
            resizable = False,
            keydown_handler = self.OnKeyDown
            )
        self.app = App(
            self.hwnd,
            vs = "2.0",
            ps = "2.0",
            fullscreen=self.fullScreen
            )
        View.winW = self.winW
        View.winH = self.winH
        View.app = self.app
        self.scene = Scene(self.app)
        self.sceneframe = Scene(self.app)
        self.sceneframe.SetScale(self.winsize)

        self.overtex = Texture('data/title/gameover.png')
        
    def OnKeyDown( self, vk, mod ):
        if vk == VK_ESCAPE :
            DestroyWindow(self.hwnd)
        elif vk == ord('L') :
            self.Debug()
    
    def Debug(self):
        #print self.app.GetDebugInfo()
        def Dump( obj, indent ):
            print " " * indent, obj
            for i in xrange( obj.NumChildren() ):
                Dump( obj.Child(i), indent + 4 )
        Dump( self.scene, 0 )

    def createView(self, camroot, frameroot, objroot):
        self.view = View(camroot, frameroot, objroot)

    def destroyView(self):
        self.view.destroy()

    def getCam(self):
        return self.view.cam

    def createTitle(self):
        self.title = Title(self.app, self.winsize)

    def createLoading(self):
        self.title = Loading(self.app, self.winsize)

    def createOver(self, status):
        self.title = Over(self.app, self.winsize, self.overtex, status)

    def destroyTitle(self):
        self.title.destroy()

class View(object):
    def __init__(self, camroot, frameroot, objroot):
        # カメラ
        self.cam = Camera(camroot)
        self.cam.SetPos(v( 0,0,-500 ))
        self.cam.SetDir(v(0,0,1),v(0,1,0))
        # カメラ（フレーム用）
        self.camfr = Camera(frameroot)
        self.camfr.SetPos(v( 0,0,-100 ))
        self.camfr.LookAt(frameroot)
        # ライト
        self.dlight = Light(objroot, LIGHT_TYPE_DIRECTIONAL)
        self.dlight.SetPos(v(300,300,-300))
        self.dlight.LookAt(objroot)
        # ゲーム画面 4:3 に３：４
        self.view = Viewport(self.app , self.cam)
        wv = int(self.winH * 3.0 / 4.0)
        xv = int((self.winW - wv) / 2.0)
        self.view.SetRect(xv, 0, wv, self.winH)
        self.view.SetFitToWindow(False)
        self.view.SetFarPlane(5000)
        self.view.SetNearPlane(10)
        self.view.SetFov( 3.1415 * 2.0 * 35.0 / 360.0 )
        self.view.SetColor(v(0.3,0.3,0.0))
        self.view.SetFillBG(True)
        # フレーム
        self.viewFr = Viewport(self.app , self.camfr)
        self.viewFr.SetRect(xv, 0, wv, self.winH)
        self.viewFr.SetFitToWindow(False)
        self.viewFr.SetFarPlane(5000)
        self.viewFr.SetNearPlane(10)
        self.viewFr.SetDrawPriority(2)
        self.viewFr.SetType(VIEWPORT_TYPE_ORTHO)
        self.viewFr.SetFillBG(False)

    def destroy(self):
        self.view.Destroy()
        self.viewFr.Destroy()

class Loading(object):
    def __init__(self, app, size):
        self.menuscene = Scene(app)
        self.menuscene.SetScale(size)
        self.menucam = Camera(self.menuscene)
        self.menucam.SetPos( v(0,0,-300) )
        self.menucam.LookAt(self.menuscene)
        self.menuplane = Plane(self.menuscene)
        self.menuplane.SetTexture(Texture('data/title/title.jpg'))
        self.menuplane.SetScale( v(1600, 1200, 1) )
        self.menuview = Viewport(app , self.menucam)
        self.menuview.SetType(VIEWPORT_TYPE_ORTHO)
        self.menuview.SetColor(v(0.0,0.0,0.0))
        self.menuview.SetFillBG(True)
        
        app.Update(0.0166)

    def destroy(self):
        self.menuview.Destroy()
        self.menuscene.Destroy()

class Title(object):
    """
    TODO: エフェクトやらなんやら
    """
    def __init__(self, app, size):
        self.menuscene = Scene(app)
        self.menuscene.SetScale(size)
        self.menucam = Camera(self.menuscene)
        self.menucam.SetPos( v(0,0,-300) )
        self.menucam.LookAt(self.menuscene)
        self.menuplane = Plane(self.menuscene)
        self.menuplane.SetTexture(Texture('data/title/title.jpg'))
        self.menuplane.SetScale( v(1600, 1200, 1) )
        self.startplane = Plane(self.menuscene)
        self.startplane.SetTexture(Texture('data/title/start.dds'))
        self.startplane.SetPos( v(0, -300, 0) )
        self.startplane.SetScale( v(352, 96, 1) )
        self.menuview = Viewport(app , self.menucam)
        self.menuview.SetType(VIEWPORT_TYPE_ORTHO)
        self.menuview.SetColor(v(0.0,0.0,0.0))
        self.menuview.SetFillBG(True)

    def destroy(self):
        self.menuview.Destroy()
        self.menuscene.Destroy()

class Over(object):
    """
    TODO:
    """
    def __init__(self, app, size, tex, status):
        self.menuscene = Scene(app)
        self.menuscene.SetScale(size)
        self.menucam = Camera(self.menuscene)
        self.menucam.SetPos( v(0,0,-500) )
        self.menucam.LookAt(self.menuscene)
        self.menuview = Viewport(app , self.menucam)
        self.menuview.SetType(VIEWPORT_TYPE_ORTHO)
        self.menuview.SetDrawPriority(10)
        
        self.menuplane = Plane(self.menuscene)
        self.menuplane.SetTexture(tex)
        self.menuplane.SetScale(v(900, 1200, 1))
        self.menuplane.SetPos(v(0, 0, 0))
        self.menuplane.SetColor(v(1,1,1,0))
        self.menuplane.AnimColor(0.2, v(1,1,1,1))
        self.score = Label(self.menucam)
        self.score.SetPos(v(0, 25, -5))
        self.score.SetColor(v(1, 1, 1, 1))
        self.score.SetFont(Font( (u'MS UI Gothic').encode('mbcs'), 30 ))
        txt = u"score:%i \nmax chain:%i \n" % (status.score, status.maxchain)
        self.score.SetText(txt)

    def destroy(self):
        self.menuview.Destroy()
        self.menuscene.Destroy()

class Score(object):
    """
    ハイスコア表示
    """
    pass
