# -*- coding: utf_8 -*-
"""
東方氷禄弾 ～ Incremental Chilled Essence ～
メイン
"""
import psyco
psyco.full()

from time import clock

from eff import *

import init
import resource
import stick
import stage
import splite
import model
import sound
import bg
import frame
import status
import achievement

DEBUG = False

class DebugMes(object):
    def __init__(self, cam):
        self.maxft = 0
        self.oldt = 0.0
        self.debug = Label(cam)
        self.debug.SetPos(v(0, 25, 100))
        self.debug.SetColor(v(0, 0, 0, 0.6))
        self.debug.SetFont(Font( (u'MS UI Gothic').encode('mbcs'), 14 ))
        self.debug.SetLayer(-10)

    def start(self):
        self.ft = clock()

    def update(self, p):
        cloudnum, icenum, itemnum = p.spMan.getSplitenum()
        t = clock()
        ft = t - self.ft
        if ft > self.maxft:
            self.maxft = ft
        fps = 1 / (t - self.oldt)
        self.oldt = t
        txt = u"FPS:%2.1f \nFT:%.6f \nMAXFT:%.6f \nCloud:%i \nIce:%i \nItem:%i \n弾:%i \nsize:%i \nscore:%i \nchain:%i \nitem:%i \n" % (
                fps, ft, self.maxft, cloudnum, icenum, itemnum, p.status.player.bullet, p.status.player.root.BoIsize
                ,p.status.score, p.status.chain, p.status.maxitem)
        self.debug.SetText(txt)
        #if fps < 55.0:
        #    print "FPS:%2.1f \nSplite:%i" % (fps, (cloudnum + icenum + itemnum))

class Main(object):
    def __init__(self):
        self.initSys = init.InitSys("system")
        self.initSys.createLoading()
        self.stk = stick.Stick(self.initSys.app, "key")
        splite.stk = self.stk
        self.bgmMan = sound.BgmMan()
        self.bgmMan.play('title')
        resource.init()
        self.seMan = sound.SeMan()
        self.status = status.Status()
        self.achievement = achievement.Achievement()
        # 変数初期化
        self.localframe = 0
        self.stage = self.menu
        self.Fpause = False
        self.initSys.destroyTitle()
        self.initSys.createTitle()
        MessageLoop(self.idle)

    def initgame(self): # game初期化
        # root
        self.objroot = Object(self.initSys.scene)
        self.bgroot = Object(self.initSys.scene)
        self.camroot = Object(self.initSys.scene)
        self.frameroot = Object(self.initSys.sceneframe)
        # マネージャーども
        self.stMan = stage.stageMan()
        self.bgMan = bg.BgMan(self.bgroot)
        self.mlMan = model.ModelMan(self.objroot)
        self.spMan = splite.SpliteMan(self.mlMan)
        self.status.player = self.spMan.ctPlayer
        self.frMan = frame.FrameMan(self.frameroot, self.status, self.spMan.wind)
        # ビュー切り替え
        self.initSys.destroyTitle()
        self.initSys.createView(self.camroot, self.frameroot, self.objroot)
        # デバッグ表示
        if DEBUG:self.debug = DebugMes(self.initSys.getCam())
        # その他初期化
        self.eventTable()
        self.localframe = 0
        self.stage = self.game
        self.bgmMan.play('otenba')

    def idle(self):
        self.stk.update()
        if self.stage == self.game and self.stk.cancelUp:
            self.Fpause = not self.Fpause
        if self.Fpause:
            self.initSys.app.Update(0)
        else:
            self.stage()
            self.localframe += 1
            self.initSys.app.Update(0.01666)

    def menu(self):
        if self.stk.okUp:
            self.initgame()
    
    def eventTable(self):
        self.etbl = {}
        self.etbl['shot'] = self.spMan.createShot
        self.etbl['cloud'] = self.spMan.createCloud
        self.etbl['ice'] = self.spMan.createIce
        self.etbl['chain'] = self.spMan.createChain
        self.etbl['item'] = self.spMan.createItem
        self.etbl['bomb'] = self.spMan.createBomb
        self.etbl['se'] = self.seMan.play
        self.etbl['wind'] = self.spMan.setWind
        self.etbl['score'] = self.status.addScore
        self.etbl['addchain'] = self.status.addChain
        self.etbl['additem'] = self.status.addItem
        self.etbl['setcombo'] = self.status.setCombo
        self.etbl['achievement'] = self.achievement.check
        self.etbl['frameachievement'] = self.frMan.setAchievement
        self.etbl['dead'] = self.dead

    def game(self):
        if DEBUG: self.debug.start()
        event = []
        self.stMan.update(event)
        self.spMan.update(self.localframe, event)
        self.bgMan.update()
        self.status.update(event)
        self.achievement.update(event)
        self.frMan.update()
        for i in event:
            self.etbl[i['type']](i['args'])
        if DEBUG: self.debug.update(self)

    def dead(self, args):
        self.frMan.over()
        self.stage = self.gameover

    def gameover(self):
        if self.stk.okUp:
            self.initSys.destroyView()
            self.frameroot.Destroy()
            self.camroot.Destroy()
            self.bgroot.Destroy()
            self.objroot.Destroy()
            del self.frMan 
            del self.spMan 
            del self.bgMan 
            del self.stMan 
            del self.mlMan 
            self.status.save()
            self.achievement.reset() # TODO save
            self.initSys.createTitle()
            self.stage = self.menu
            self.bgmMan.play('title')


Main()
