# -*- coding: utf_8 -*-
"""
モデル
"""

from eff import *
import resource
from util import *

res = None

class ModelMan(object):
    def __init__(self, parent):
        global res
        res = resource.res
        self.parent = parent

    def create(self, type):
        return modelTbl[type](self.parent)

# MixIn
class PlaneMixin(object):
    def setPlane(self, m, parent, scale, fx = ('plane', 'Tech_Alpha'), pos = V0, dir = (VZ, VY)):
        plane = Plane(parent)
        plane.SetTexture(res.tex[m])
        plane.SetEffect(res.fx[fx[0]], fx[1])
        plane.SetScale(scale)
        plane.SetDir(dir[0], dir[1])
        plane.SetPos(pos)
        return plane

class ModelMixin(object):
    def setModel(self, m, parent, fx = ('bone', 'TechNormal'), pos = V0, dir = (VY, -VZ), scale = V1):
        model = Model(parent)
        model.SetModel(res.model[m])
        model.SetEffect(res.fx[fx[0]], fx[1])
        model.SetPos(pos)
        model.SetScale(scale)
        model.SetDir(dir[0], dir[1])
        return model

    def initTrack(self, obj, namelist):
        return dict(zip(namelist, [obj.FindTrack(i) for i in namelist]))

    def setTrack(self, obj, track, weight, loop = True):
        obj.SetTrackEnable(track, True);
        obj.SetTrackWeight(track, weight);
        obj.SetTrackSpeed(track, 1.0);
        obj.SetTrackLoop(track, loop);

    def changeTrack(self, obj, trackN, track, time = 0.0, mode = ''):
        """
        指定したトラックのウエイトを１、それ以外を０に
        """
        for i in trackN.values():
            obj.AnimTrackWeight(time, i, 0.0);
        obj.AnimTrackWeight(time, track, 1.0);
        if mode == 'reset':
            obj.SetTrackPos(track, 0.0);

    def fadeTrack(self, obj, track, wait, time):
        """
        指定したトラックのウエイトを設定
        """
        obj.AnimTrackWeight(time, track, wait);

    def setTrackAnime(self, type):
        pass

class ParticleMixin(object):
    def setParticle(self, m, parent, anim, fx = ('particle', 'Tech_Alpha'), pos = V0, size = V1, dir = (VZ, VY)):
        pt = Particle(self.root)
        pt.SetTexture(res.tex[m])
        pt.SetEffect(res.fx[fx[0]], fx[1])
        pt.SetAnimScript(res.anim[anim[0]], anim[1])
        pt.SetScale(size)
        return pt

# base
class ModelB(object):
    def __init__(self, parent):
        self.parent = parent
        self.root = Object(parent)
        self.localframe = 0
        self.kill = self.root.Destroy

    def setAnim(self, type, args):
        self.root.SetAnimScript(res.anim[type], args)

    def exp(self):
        pass

    def setPos(self, p):
        self.root.SetPos(p)
     
    def getPos(self):
        return self.root.GetPos()
     
    pos = property(getPos,setPos,None,None)

    def checkRange(self):
        pos = self.root.GetPos()
        return not (-SCREENW2 < pos.x < SCREENW2 and -SCREENH2 < pos.y < SCREENH2)

    def checkRange2(self): # 外周
        pos = self.root.GetPos()
        if vlen(pos) > 300:
            return True

    def update(self, frame):
        self.localframe += 1

# player
class ModelPlayer(ModelB, PlaneMixin, ModelMixin, ParticleMixin):
    def __init__(self, parent):
        ModelB.__init__(self, parent)
        #
        self.BoIsize = 180.0
        self.Fmove = 0
        # モーション初期化
        namelist = ('main', 'right', 'left', 'front', 'back', 'tama', 'loose')
        self.model = self.setModel('tiruno', self.root, pos = VY * -5.0)
        self.modelTr = self.initTrack(self.model, namelist)
        for type in namelist:
            self.setTrack(self.model, self.modelTr[type], 0.0)
        self.setTrack(self.model, self.modelTr['main'], 1.0)
        self.setTrack(self.model, self.modelTr['tama'], 0.0, False)
        # BoI　枠
        self.modelBoI = self.setPlane('BoI', self.root, V0, fx = ('plane', 'Tech_Add'), pos = VZ * -15.0)
        self.modelBoI.SetAnimScript(res.anim['rotm'], {'rot':0.03})
        # BoI　パーティクル
        self.particleBoI = self.setParticle('BoIpt', self.root, ('boip',{'size':self.BoIsize}),
                fx = ('particle', 'Tech_Add'), size = V0)
        # センター表示
        self.modelCenter = self.setPlane('center', self.root, V1 * 60.0, fx = ('plane', 'Tech_Add'), pos = VZ * -15.0)
        self.modelCenter.SetAnimScript(res.anim['rotm'], {'rot':0.05})

    def left(self):
        if self.Fmove >= 0:
            self.changeTrack(self.model, self.modelTr, self.modelTr['left'], time = 0.2)
            self.Fmove = -1

    def center(self):
        if self.Fmove != 0:
            self.changeTrack(self.model, self.modelTr, self.modelTr['main'], time = 0.2)
            self.Fmove = 0

    def right(self):
        if self.Fmove <= 0:
            self.changeTrack(self.model, self.modelTr, self.modelTr['right'], time = 0.2)
            self.Fmove = 1

    def shotCstart(self):
        self.changeTrack(self.model, self.modelTr, self.modelTr['tama'], 0.05, 'reset')

    def shotCend(self):
        self.fadeTrack(self.model, self.modelTr['tama'], 0.0, 0.1)
        # TODO とりあえずmainモーションで
        self.fadeTrack(self.model, self.modelTr['main'], 1.0, 0.1)

    def dead(self):
        # 自機モデル
        #self.root.AnimPos(1.5, self.root.GetPos() - VY * 50.0)
        self.root.AnimPos(1.5, self.root.GetPos() - VY * 300.0)
        self.changeTrack(self.model, self.modelTr, self.modelTr['loose'], 0.05, 'reset')
        self.model.AnimScale(1.5, V0)
        self.model.AnimColor(1.5, C0)
        # BoI割れるモデルに差替
        s = self.modelBoI.GetScale()
        self.modelBoI.Destroy()
        obj = self.setModel('BoI2', self.root, fx = ('bone', 'TechAdd2'), pos = VZ * -15.0, scale = s)
        obj.SetModel(res.model['BoI2'])
        Tr = self.initTrack(obj, ('anim',))
        self.setTrack(obj, Tr['anim'], 1.0, False)
        obj.AnimColor(1.0, C0)
        # 雪のパーティクル
        pt = self.setParticle('yuki1', self.root, ('boip2', {}), fx = ('particle', 'Tech_Add'))
        # その他を消す
        self.particleBoI.AnimScale(0.4, V0)
        self.particleBoI.AnimColor(0.4, C0)
        self.modelCenter.AnimScale(1.0, V0)
        self.modelCenter.AnimColor(1.0, C0)

    def on(self):
        self.modelBoI.AnimScale(1.5, V1 * self.BoIsize)
        self.particleBoI.AnimScale(1.8, V1)
        self.particleBoI.SendMessage(0,(self.BoIsize,))

    def off(self):
        self.modelBoI.AnimScale(0.3, V0)
        self.particleBoI.AnimScale(0.25, V0)

    def add(self):
        """ TODO:最大径でBoIに変化
        """
        if self.BoIsize < 240.0:
            self.BoIsize += 10.0
        else:
            self.BoIsize = 100.0


    def getSize(self):
        v = self.modelBoI.GetScale()
        return v.x / 2.0

# shot
class ShotMixin(object):
    def dead(self):
        # TODO エフェクト追加
        self.Fdead = True

    def update(self, frame):
        ModelB.update(self, frame)
        return self.Fdead

class ModelShot(ShotMixin, ModelB, ModelMixin):
    def __init__(self, parent):
        ModelB.__init__(self, parent)
        #
        self.root.SetDir(VY, -VZ)
        self.model = self.setModel('shot', self.root, fx = ('ice', 'Tech_Ice'), scale = V1 * 2.0)
        self.model.SetAnimScript(res.anim['roti'], {'rot':0.3})
        self.Fdead = False

class ModelShotB(ShotMixin, ModelB, ModelMixin):
    def __init__(self, parent):
        ModelB.__init__(self, parent)
        #
        self.root.SetDir(-VY, VZ)
        self.model = self.setModel('shot', self.root, fx = ('ice', 'Tech_Ice'), scale = V1 * 2.0)
        self.model.SetAnimScript(res.anim['roti'], {'rot':0.3})
        self.Fdead = False

class ModelShotC(ShotMixin, ModelB, ModelMixin):
    """
    大きくなってから発射
    """
    def __init__(self, parent):
        ModelB.__init__(self, parent)
        #
        #self.root.SetDir(VY, -VX)
        self.model = self.setModel('bigice', self.root, fx = ('ice', 'Tech_Ice'))
        self.Fdead = False

# cloud
class ModelCloud(ModelB, PlaneMixin):
    def __init__(self, parent, m):
        ModelB.__init__(self, parent)
        size = uniform(70.0, 100.0) * V1
        type = randint(0, 3)
        dz = (5.0, 5.0, 10.0, 15.0)[type]
        color = (C1, C1, v(0.95, 0.95, 0.95, 1.0), v(0.9, 0.9, 0.9, 1.0))[type]
        p = VZ * uniform(dz, dz + 5.0)
        self.model = self.setPlane(m, self.root, size, fx = ('plane', 'Tech_Alpha_OC'), pos = p, dir = rndDir())
        self.model.SetColor(color)
        self.model.SetLayer(50)
        
    def exp(self):
        self.model.AnimScale(0.2, V0)

def ModelCloudW(p):
    return ModelCloud(p, 'cloudW')

def ModelCloudR(p):
    return ModelCloud(p, 'cloudR')

def ModelCloudG(p):
    return ModelCloud(p, 'cloudG')

def ModelCloudB(p):
    return ModelCloud(p, 'cloudB')

def ModelCloudY(p):
    return ModelCloud(p, 'cloudY')

def ModelCloudP(p):
    return ModelCloud(p, 'cloudP')

# ice
class ModelIceW(ModelB, ModelMixin):
    def __init__(self, parent):
        ModelB.__init__(self, parent)
        self.model = self.setModel('iceW', self.root, fx = ('ice', 'Tech_Ice'), scale = V1 * 1.3)
        self.model.SetPos(v(0,0,uniform(-2.0, 5.0)))
        eye, up = rndDir2()
        self.model.SetDir(eye, up)
        self.model.SetLayer(30)

class ModelIceR(ModelB, ModelMixin):
    def __init__(self, parent):
        ModelB.__init__(self, parent)
        self.model = self.setModel('iceR', self.root, fx = ('ice', 'Tech_Ice'), scale = V1 * 0.8)
        self.model.SetPos(v(0,0,uniform(-2.0, 5.0)))
        eye, up = rndDir2()
        self.model.SetDir(eye, up)
        self.model.SetLayer(0)

class ModelIceG(ModelB, ModelMixin):
    def __init__(self, parent):
        ModelB.__init__(self, parent)
        self.model = self.setModel('iceG', self.root, fx = ('ice', 'Tech_Ice'), scale = V1 * 0.8)
        self.model.SetPos(v(0,0,uniform(-2.0, 5.0)))
        eye, up = rndDir2()
        self.model.SetDir(eye, up)
        self.model.SetLayer(0)

class ModelIceB(ModelB, ModelMixin):
    def __init__(self, parent):
        ModelB.__init__(self, parent)
        self.model = self.setModel('iceB', self.root, fx = ('ice', 'Tech_Ice'), scale = V1 * 0.8)
        self.model.SetPos(v(0,0,uniform(-2.0, 5.0)))
        eye, up = rndDir2()
        self.model.SetDir(eye, up)
        self.model.SetLayer(0)

class ModelIceY(ModelB, ModelMixin):
    def __init__(self, parent):
        ModelB.__init__(self, parent)
        self.model = self.setModel('iceY', self.root, fx = ('ice', 'Tech_Ice'), scale = V1 * 0.8)
        self.model.SetPos(v(0,0,uniform(-2.0, 5.0)))
        eye, up = rndDir2()
        self.model.SetDir(eye, up)
        self.model.SetLayer(0)

class ModelIceP(ModelB, ModelMixin):
    def __init__(self, parent):
        ModelB.__init__(self, parent)
        self.model = self.setModel('iceP', self.root, fx = ('ice', 'Tech_Ice'), scale = V1 * 0.8)
        self.model.SetPos(v(0,0,uniform(-2.0, 5.0)))
        eye, up = rndDir2()
        self.model.SetDir(eye, up)
        self.model.SetLayer(0)

# item
class ModelItemW(ModelB, ModelMixin):
    def __init__(self, parent):
        ModelB.__init__(self, parent)
        self.model = self.setModel('itemW', self.root, fx = ('item', 'Tech_Ice'))
        self.model.SetPos(v(0,0,uniform(6.05, 6.15)))
        self.model.SetAnimScript(res.anim['roti'], {'rot':0.05})
        self.model.SetLayer(40)

class ModelItemW2(ModelB, ModelMixin):
    def __init__(self, parent):
        ModelB.__init__(self, parent)
        self.model = self.setModel('itemW', self.root, fx = ('item', 'Tech_Ice'), scale = V1 * 2.0)
        self.model.SetPos(v(0,0,uniform(6.05, 6.15)))
        self.model.SetAnimScript(res.anim['roti'], {'rot':0.05})
        self.model.SetLayer(40)

class ModelItemW3(ModelB, ModelMixin):
    def __init__(self, parent):
        ModelB.__init__(self, parent)
        self.model = self.setModel('itemW', self.root, fx = ('item', 'Tech_Ice'), scale = V1 * 3.0)
        self.model.SetPos(v(0,0,uniform(6.05, 6.15)))
        self.model.SetAnimScript(res.anim['roti'], {'rot':0.05})
        self.model.SetLayer(40)

class ModelItemW4(ModelB, ModelMixin):
    def __init__(self, parent):
        ModelB.__init__(self, parent)
        self.model = self.setModel('itemW', self.root, fx = ('item', 'Tech_Ice'), scale = V1 * 4.0)
        self.model.SetPos(v(0,0,uniform(6.05, 6.15)))
        self.model.SetAnimScript(res.anim['roti'], {'rot':0.05})
        self.model.SetLayer(40)

class ModelItemR(ModelB, ModelMixin):
    def __init__(self, parent):
        ModelB.__init__(self, parent)
        self.model = self.setModel('itemR', self.root, fx = ('item', 'Tech_Ice'))
        self.model.SetPos(v(0,0,uniform(6.05, 6.15)))
        self.model.SetAnimScript(res.anim['roti'], {'rot':0.05})
        self.model.SetLayer(40)

class ModelItemB(ModelB, ModelMixin):
    def __init__(self, parent):
        ModelB.__init__(self, parent)
        self.model = self.setModel('itemB', self.root, fx = ('item', 'Tech_Ice'))
        self.model.SetPos(v(0,0,uniform(6.05, 6.15)))
        self.model.SetAnimScript(res.anim['roti'], {'rot':0.05})
        self.model.SetLayer(40)

class ModelItemG(ModelB, ModelMixin):
    def __init__(self, parent):
        ModelB.__init__(self, parent)
        self.model = self.setModel('itemG', self.root, fx = ('item', 'Tech_Ice'))
        self.model.SetPos(v(0,0,uniform(6.05, 6.15)))
        self.model.SetAnimScript(res.anim['roti'], {'rot':0.05})
        self.model.SetLayer(40)

class ModelItemY(ModelB, ModelMixin):
    def __init__(self, parent):
        ModelB.__init__(self, parent)
        self.model = self.setModel('itemY', self.root, fx = ('item', 'Tech_Ice'))
        self.model.SetPos(v(0,0,uniform(6.05, 6.15)))
        self.model.SetAnimScript(res.anim['roti'], {'rot':0.05})
        self.model.SetLayer(40)

class ModelItemP(ModelB, ModelMixin):
    def __init__(self, parent):
        ModelB.__init__(self, parent)
        self.model = self.setModel('itemP', self.root, fx = ('item', 'Tech_Ice'))
        self.model.SetPos(v(0,0,uniform(6.05, 6.15)))
        self.model.SetAnimScript(res.anim['roti'], {'rot':0.05})
        self.model.SetLayer(40)

# chain
class ModelChain(ModelB, PlaneMixin): # モデル無しかパーティクルか
    def __init__(self, parent):
        ModelB.__init__(self, parent)
        self.model = self.setPlane('chain', self.root, V1 * 10, fx = ('plane', 'Tech_Add'))
        eye, up = rndDir()
        self.model.SetDir(eye, up)
        dt = 0.12
        self.model.AnimScale(dt, V1 * 50.0)
        self.model.AnimColor(dt, CA)

class ModelBomb(ModelB, PlaneMixin, ParticleMixin):
    def __init__(self, parent):
        ModelB.__init__(self, parent)
        self.model = self.setPlane('bomb', self.root, V0, fx = ('plane', 'Tech_Add_OC'))

    def set(self, size):
        self.model.AnimScale(0.2, V1 * size)
        self.model.AnimColor(1.0, C0)
        self.particleBoI = self.setParticle('BoIpt', self.root, ('bombpt',{'size':size}),
                fx = ('particle', 'Tech_Add'))

modelTbl = {
    'root':ModelB,
    'player':ModelPlayer,
    'shot':ModelShot,
    'shotB':ModelShotB,
    'shotC':ModelShotC,
    'chain':ModelChain,
    'bomb':ModelBomb,
    'cloudW':ModelCloudW,
    'iceW':ModelIceW,
    'cloudR':ModelCloudR,
    'iceR':ModelIceR,
    'cloudG':ModelCloudG,
    'iceG':ModelIceG,
    'cloudB':ModelCloudB,
    'iceB':ModelIceB,
    'cloudY':ModelCloudY,
    'iceY':ModelIceY,
    'cloudP':ModelCloudP,
    'iceP':ModelIceP,
    'chain':ModelChain,
    'itemW':ModelItemW,
    'itemW2':ModelItemW2,
    'itemW3':ModelItemW3,
    'itemW4':ModelItemW4,
    'itemR':ModelItemR,
    'itemB':ModelItemB,
    'itemG':ModelItemG,
    'itemY':ModelItemY,
    'itemP':ModelItemP,
    }

