# -*- coding: utf_8 -*-

from eff import *
from actor import *
from util import *

SHOTHIT = 15.0
SHOTSPEED = VY * 6.0
SPEED = 2.1

class SplitePlayer(Space):# 自機
    def __init__(self, stk):
        self.stk = stk
        self.root = ml.create('player')
        self.pos = -VY * 100.0
        self.root.pos = self.pos
        self.reloading = 0 
        self.nohitTime = 0.0
        self.mode = 0 # 弾の種類
        self.Fnohit = False
        self.Fboi = False
        self.shotcnt = 0
        self.bullet = 9 # 弾数
        self.localframe = 0
        self.Fnormalmotion = True # 通常モーション時
        self.ev = Event(())
        self.task = 'loop'
        self.taskTbl = {
            'loop': self.taskloop,
            'shotCstart': self.taskshotCstart,
            'shotCend': self.taskshotCend,
            'dead': self.taskdead,
            }
    
    def checkShot(self):
        b = (1, 2, 5)[self.mode]
        if self.reloading <= 0 and self.bullet >= b:
            self.reloading = (15, 20, 200)[self.mode]
            self.bullet -= b
            if self.Fboi:
                self.root.off()
                self.Fboi = False
            if self.mode == 2:
                self.ev.add('task', 'shotCstart', 0)
                self.ev.add('task', 'shotCend', 170)
            return True

    def shotMode(self, mode, que):
        self.bullet += (1, 2, 5)[mode]
        if self.bullet >= 31:
            sc = (self.bullet - 30) * 10
            que.append({'type':'score', 'args':sc})
            self.bullet = 30
        self.mode = mode
    
    def boi(self):
        self.root.add()
        if self.Fboi:
            self.root.on()

    def getHitrange(self):
        return (self.pos, 2.0, 1)

    def getBoIrange(self):
        return (self.pos, self.root.getSize(), 1)

    def getRoot(self):
        return self.root.root
    
    def dead(self):
        self.Fnohit = True
        self.ev.clear()
        self.ev.add('task', 'dead', 180)
        self.root.dead()
    
    def move(self):
        axis = self.stk.axis
        if self.Fnormalmotion:
            if axis.x < 0.0:
                self.root.left()
            elif axis.x > 0.0:
                self.root.right()
            else:
                self.root.center()
        x = self.pos.x + axis.x * SPEED 
        y = self.pos.y + axis.y * SPEED 
        if -SCREENW > x:
            x = -SCREENW
        elif SCREENW < x:
            x = SCREENW
        if -SCREENH > y:
            y = -SCREENH
        elif SCREENH < y:
            y = SCREENH
        self.pos = v(x, y, 0.0)
        self.root.pos = v(x, y, 0.0)

    def taskloop(self, que):
        pass

    def taskshotCstart(self, que):
        self.Fnormalmotion = False
        self.root.shotCstart()
        self.task = 'loop'

    def taskshotCend(self, que):
        self.Fnormalmotion = True
        self.root.shotCend()
        self.task = 'loop'

    def taskdead(self, que):
        que.append({'type':'dead','args':None})
        self.task = 'loop'
    
    def update(self, frame, que):
        self.localframe += 1
        # input
        if not self.Fnohit:
            if self.stk.ok and self.checkShot():
                self.shotcnt += 1
                que.append({'type':'shot','args':None})
                que.append({'type':'se', 'args':'myshot'})
                que.append({'type':'achievement', 'args':{'type':'shotcnt', 'param': self.shotcnt}})
                que.append({'type':'achievement', 'args':{'type':'shotmode', 'param': self.mode}})
            self.move()
            #
            self.root.update(frame)
            self.reloading -= 1
            if self.reloading < 0 and not self.Fboi:
                self.root.on()
                self.Fboi = True
            if self.localframe % 1800 == 0: # 弾数自動増加
                self.bullet += (1, 2, 5)[self.mode]
                if self.bullet >= 31:
                    self.bullet = 30
        for i in self.ev:
            if i[0] == 'task':
                self.task = i[1]
        self.taskTbl[self.task](que)

class SpliteShot(Space):# 自弾
    def __init__(self, player):
        self.root = ml.create('shot')
        self.root.pos = player.pos
        self.Fhit = 0
        self.root.setAnim('liner' , {"spd":SHOTSPEED})

    def getHitrange(self):
        return (self.root.pos, SHOTHIT, 1)
    
    def update(self, frame, que):
        if self.root.pos.y > 150 and self.Fhit == 0:
            self.Fhit = 1
        if self.Fhit == 1:
            self.root.dead()
            self.Fhit = 2
        if self.Fhit == 2: # エフェクト終了待ち
            if self.root.update(frame):
                self.root.kill()
                return True

class SpliteShotB(Space):# 自弾
    def __init__(self, player):
        self.root = ml.create('shotB')
        self.root.pos = player.pos
        self.Fhit = 0
        self.root.setAnim('liner' , {"spd":-SHOTSPEED})
    
    def getHitrange(self):
        return (self.root.pos, SHOTHIT, 1)
    
    def update(self, frame, que):
        if self.root.pos.y < -150 and self.Fhit == 0:
            self.Fhit = 1
        if self.Fhit == 1:
            self.root.dead()
            self.Fhit = 2
        if self.Fhit == 2: # エフェクト終了待ち
            if self.root.update(frame):
                self.root.kill()
                return True

class SpliteShotC(Space):# 自弾
    def __init__(self, player):
        self.root = ml.create('shotC')
        self.root.pos = player.pos
        self.Fhit = 0
        self.root.setAnim('shotC' , {"spd":3.0, "player":player.getRoot()})
        self.localframe = 0
    
    def getHitrange(self):
        damage = 10 if self.localframe > 40 else 0 # タメ中　どうしたもんか
        return (self.root.pos, 30, damage)
    
    def update(self, frame, que):
        self.localframe += 1
        if self.root.pos.y > 150:
            self.root.dead()
            self.Fhit = 2
        if self.Fhit == 2:
            if self.root.update(frame):
                self.root.kill()
                return True

class SpliteBomb(object):# ボム
    def __init__(self, player):
        self.root = ml.create('bomb')
        self.root.set(player.root.BoIsize)
        self.pos = player.pos
        self.root.pos = self.pos
        self.localframe = 0
    
    def getHitrange(self):
        r = self.root.model.GetScale().x / 2.0
        return (self.pos, r, 1)
    
    def update(self, frame, que):
        self.localframe += 1
        if self.localframe > 1.0 * 60.0:
            self.root.kill()
            return True

