# -*- coding: utf_8 -*-
"""
スプライト管理
"""
from eff import *

import player
import enemy
from util import *


def spaceindex(n): # 分割領域の参照用
    return [[x + dx + (y + dy) * n for dy in (-1, 0, 1) for dx in (-1, 0, 1)
                if 0 <= x + dx < n and 0 <= y + dy < n]
            for y in range(n) for x in range(n)]

class Wind(object):# 風向き
    def __init__(self, parent):
        self.root = Object(parent)
        self.root.SetPos(-VY)
        self.changeDir(0, 0)
        self.spd = 0
    
    def changeDir(self, area, spd):
        """
        area 0,1,2,3
        そのエリア内でランダムに（だいたい30度ぐらいか）
        spd 0~3 速度はrank固定 > stage
        """
        self.spd = spd
        r = ((150, 210), (60, 120), (-30,30), (240, 300))[area]
        dir = rotRndZ(r[0], r[1])
        s = ((0.3, 0.5), (0.7, 1.0), (1.3, 2.0), (2.8, 3.5))[spd]
        dir *= uniform(s[0], s[1])
        dt = randint(2, 4)
        self.root.AnimPos(dt, dir)

    def getRoot(self):
        return self.root
 
class SpliteMan(object):
    def __init__(self, mlMan):
        player.ml = mlMan
        enemy.ml = mlMan
        self.wind = Wind(mlMan.parent)
        enemy.wind = self.wind.getRoot()
        self.cnt = 0
        self.ctPlayer = player.SplitePlayer(stk)
        self.ctShot = {}
        self.ctBomb = {}
        self.ctCloud = {}
        self.ctIce = {}
        self.ctChain = {}
        self.ctItem = {}
        self.ctBombtarget = (self.ctCloud, self.ctIce)
        self.ctAll = (self.ctShot, self.ctCloud, self.ctIce, self.ctChain, self.ctItem, self.ctBomb)
        self.spaceidx = spaceindex(5)
    
    def getSplitenum(self):
        return len(self.ctCloud), len(self.ctIce), len(self.ctItem)

    def createShot(self, args):
        mode = self.ctPlayer.mode
        if mode == 0: # normal
            self.ctShot[self.cnt] = player.SpliteShot(self.ctPlayer)
            self.cnt += 1
        elif mode == 1: # tail
            self.ctShot[self.cnt] = player.SpliteShot(self.ctPlayer)
            self.cnt += 1
            self.ctShot[self.cnt] = player.SpliteShotB(self.ctPlayer)
            self.cnt += 1
        elif mode == 2: # big
            self.ctShot[self.cnt] = player.SpliteShotC(self.ctPlayer)
            self.cnt += 1
  
    def createBomb(self, args):
        self.ctBomb[self.cnt] = player.SpliteBomb(self.ctPlayer)
        self.cnt += 1

    def createCloud(self, args):
        args['player'] = self.ctPlayer
        self.ctCloud[self.cnt] = enemy.spliteEnTbl[args['type']](args)
        self.cnt += 1

    def createIce(self, args):
        args['player'] = self.ctPlayer
        self.ctIce[self.cnt] = enemy.spliteEnTbl[args['type']](args)
        self.cnt += 1

    def createChain(self, args):
        args['player'] = self.ctPlayer
        self.ctChain[self.cnt] = enemy.spliteEnTbl['chain'](args)
        self.cnt += 1

    def createItem(self, args):
        args['player'] = self.ctPlayer
        self.ctItem[self.cnt] = enemy.spliteEnTbl[args['type']](args)
        self.cnt += 1

    def setWind(self, args):
        self.wind.changeDir(args['area'], args['spd'])

    def update(self, frame, que):
        # hittest
        # BoI + cloud
        if self.ctPlayer.reloading <= 0 and not self.ctPlayer.Fnohit: # 非攻撃時のみ氷化
            r = self.ctPlayer.getBoIrange()
            for obj2 in self.ctCloud.values():
                obj2.hitTest2(r)
        # bomb + target
        for obj in self.ctBomb.values():
            r = obj.getHitrange()
            for c in self.ctBombtarget:
                for obj2 in c.values():
                    obj2.bomb(r)
        # shot + cloud
        shotmode = self.ctPlayer.mode != 2 # 貫通弾対策
        for obj in self.ctShot.values():
            if obj.Fhit >= 1:
                break
            r = obj.getHitrange()
            for obj2 in self.ctCloud.values():
                if obj2.hitTest(r):
                    if shotmode:
                        obj.Fhit = 1
                        break
        # ice space
        sp = [[] for i in xrange(25)]
        for i in self.ctIce.values():
            i.setSpace(sp)
        # shot + ice
        for obj in self.ctShot.values():
            if obj.Fhit >= 1:
                break
            idx = obj.getIdx()
            if idx >= 0:
                r = obj.getHitrange()
                for i in self.spaceidx[idx]:
                    for obj2 in sp[i]:
                        if obj2.hitTest(r):
                            if shotmode:
                                obj.Fhit = 1
                                break
        # chain + ice
        for obj in self.ctChain.values():
            idx = obj.getIdx()
            if idx >= 0:
                r = obj.getHitrange()
                if r == False:
                    continue
                for i in self.spaceidx[idx]:
                    for obj2 in sp[i]:
                        obj2.hitTestChain(r)
        # player + item
        obj = self.ctPlayer
        if not obj.Fnohit:
            r = obj.getHitrange()
            for j in self.ctItem.values():
                if j.hitTest(r):
                    id = j.id
                    if 0 <= id <= 2:
                        self.ctPlayer.shotMode(id, que)
                    elif id == 3:
                        self.ctPlayer.boi()
                    elif id == 4:
                        que.append({'type':'bomb','args':None})
                    elif id == 5:
                        que.append({'type':'additem','args':None})
                    que.append({'type':'achievement', 'args':{'type':'getitem', 'param': id}})
        # player + ice
            idx = obj.getIdx()
            for i in self.spaceidx[idx]:
                for obj2 in sp[i]:
                    if obj2.hitTest(r):
                        obj.dead()
                        break
        # update
        for c in self.ctAll:
            for i in c.keys():
                if c[i].update(frame, que):
                    c.pop(i)
        self.ctPlayer.update(frame, que)


